/*
 * IR Click MBN Driver
 * 
 * Version 1.0.0
 *  - Initial release coded by Stephen Cardinale
 * Version 1.0.1
 *  - Bug Fix in the Sony Receiver class when stopping the Sony Receiver.
 *  
 * References needed :
 *   - Microsoft.SPOT.Hardware
 *   - Microsoft.SPOT.Native
 *   - MikroBus.Net
 *   - mscorlib
 *  
 * Copyright 2014 MikroBus.Net
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

using System;
using Microsoft.SPOT.Hardware;

namespace MBN.Modules
{
    /// <summary>
    /// Abstract class representing IR remote receiver
    /// </summary>
    abstract public class IrReceiver : IDisposable
    {
        /// <summary>
        /// Get or set pin used for IR demodulator
        /// </summary>
        protected Cpu.Pin ReceiverPin;

		/// <summary>
		/// Disposes of the object.
		/// </summary>
		public abstract void Dispose();

        /// <summary>
        /// Data received delegate
        /// </summary>
        /// <param name="sender">Instance where event occurred</param>
        /// <param name="command">Command received</param>
        /// <param name="address">Address received</param>
        /// <param name="time">The DateTime that the event occurred.</param>
        internal delegate void IrDataHandler(IrReceiver sender, int command, int address, DateTime time);
        
        /// <summary>
        /// Event occurs when data is received
        /// </summary>
        internal virtual event IrDataHandler DataReceived;

	    internal void OnDataReceived(int command, int address)
        {
            var handler = DataReceived;
            if (handler != null) handler(this, command, address, DateTime.Now);
        }  
    }
}
